// start
package metaheuristicOptimization.algorithms.de;

import metaheuristicOptimization.IObjectiveFunction;
import metaheuristicOptimization.Individual;
import metaheuristicOptimization.OptimizationAlgorithm;
import metaheuristicOptimization.searchSpaces.Rn.RnRecombineDE;

/**
 * the Differential Evolution algorithm
 *
 * @param <X>
 *          the solution space
 */
// end
public class DE<X> extends OptimizationAlgorithm<double[], X> { // start

  /** the PSO recombination algorithm */
  public RnRecombineDE ternary;

  /** the population size */
  public int ps; // start

  /** create */
  public DE() {
    super();
    this.ps = 128;
  }

  /**
   * Solve the optimization problem
   *
   * @param f
   *          the objective function
   * @return the best individual found
   */
  @Override
  @SuppressWarnings("unchecked")
  // end
  public Individual<double[], X> solve(final IObjectiveFunction<X> f) {
    Individual<double[], X>   pbest, pcur;
    Individual<double[], X>[] pop, ofs;
    int i, j, k;

    pbest = new Individual<>();
    pop   = new Individual[this.ps];  // the population: all elements are null
    ofs   = new Individual[this.ps];  // the set of offsprings

    for (i = ofs.length; (--i) >= 0;) {
      ofs[i] = pcur = new Individual<>();
      pcur.g = this.nullary.create(this.random);
    }

    for (;;) {
      for (i   = ofs.length; (--i) >= 0;) {
        pcur   = ofs[i];                        // pcur is the ith offspring
        pcur.x = this.gpm.gpm(pcur.g);          // do gpm
        pcur.v = f.compute(pcur.x);             // get objective value
        if ((pop[i] == null) || (pcur.v <= pop[i].v)) { // offspring better than parent?
          pop[i] = pcur;                                // replace parent
          if (pcur.v < pbest.v) {              // check if we found best
            pbest.assign(pcur);
          }
        }
        if (this.termination.shouldTerminate()) {
          return pbest;
        }
      }

      for (i = pop.length; (--i) >= 0;) {     // create offspring
        ofs[i] = pcur = new Individual<>();   // pick first parent

        do {
          j = this.random.nextInt(pop.length);
        } while (j == i); // different parents!
        do {
          k = this.random.nextInt(pop.length);
        } while ((k == i) || (k == j)); // different parents!

        pcur.g = this.ternary.recombine(pop[j].g, pop[k].g, pop[i].g, this.random);
      }
    }
  }
}