// start
package metaheuristicOptimization.algorithms.ea;

import metaheuristicOptimization.IBinarySearchOperation;
import metaheuristicOptimization.IObjectiveFunction;
import metaheuristicOptimization.Individual;
import metaheuristicOptimization.OptimizationAlgorithm;
import metaheuristicOptimization.algorithms.ea.selection.TruncationSelection;

/**
 * the evolutionary algorithm
 * 
 * @param <G>
 *          the search space
 * @param <X>
 *          the problem/solution space
 */// end
public class EA<G, X> extends OptimizationAlgorithm<G, X> {// start
  /** the temperature */// end
  public ISelectionAlgorithm selection; // start

  /** the population size */// end
  public int ps;// start

  /** the mating pool */// end
  public int mps;// start

  /** the crossover rate */// end
  public double cr;// start

  /** the binary search operation */// end
  public IBinarySearchOperation<G> binary;// start

  /** create */// end
  public EA() {
    super();
    this.cr = 0.3d;
    this.ps = 128;
    this.mps = 64;
    this.selection = TruncationSelection.INSTANCE;
  }// start

  /** {@inheritDoc} */
  @SuppressWarnings("unchecked")
  @Override
  // end
  public Individual<G, X> solve(final IObjectiveFunction<X> f) {
    Individual<G, X> pbest, pcur;
    Individual<G, X>[] pop, mate;
    int i;

    pbest = new Individual<>();
    pop = new Individual[this.ps];
    mate = new Individual[this.mps];

    for (i = pop.length; (--i) >= 0;) {
      pop[i] = pcur = new Individual<>();
      pcur.g = this.nullary.create(this.random);
    }

    for (;;) {
      for (i = pop.length; (--i) >= 0;) {
        pcur = pop[i];
        pcur.x = this.gpm.gpm(pcur.g);
        pcur.v = f.compute(pcur.x);
        if (pcur.v < pbest.v) {
          pbest.assign(pcur);
        }
        if (this.termination.shouldTerminate()) {
          return pbest;
        }
      }

      this.selection.select(pop, mate, this.random);

      for (i = pop.length; (--i) >= 0;) {
        pop[i] = pcur = new Individual<>();
        if (this.random.nextDouble() < this.cr) {
          pcur.g = this.binary.recombine(mate[i % mate.length].g,
              mate[this.random.nextInt(mate.length)].g, this.random);
        } else {
          pcur.g = this.unary.mutate(mate[i % mate.length].g, this.random);
        }
      }
    }
  }
}
