// start
package metaheuristicOptimization.algorithms.es;

import metaheuristicOptimization.IObjectiveFunction;
import metaheuristicOptimization.Individual;
import metaheuristicOptimization.OptimizationAlgorithm;
import metaheuristicOptimization.searchSpaces.Rn.RnESUnaryNormal;

/**
 * a (1+1) Evolution Strategy
 * 
 * @param <X>
 *          the solution space
 */
// end
public class ES1P1<X> extends OptimizationAlgorithm<double[], X> {// start

  /** the step-width multiplier */
  public double a;

  /** the initial sigma */
  public double sigma0;

  /** the adatation frequency */
  public int L; // start

  /** instantiate */
  public ES1P1() {
    super();
  }

  /**
   * Solve the optimization problem
   * 
   * @param f
   *          the objective function
   * @return the best individual found
   */
  @Override
  // end
  public Individual<double[], X> solve(final IObjectiveFunction<X> f) {      
    Individual<double[], X> pstar = new Individual<>(); // best individual 
    Individual<double[], X> pnew  = new Individual<>(); // "new" individual
    RnESUnaryNormal esUnary       = ((RnESUnaryNormal) (this.unary));
    
    double[] sigma = new double[] { this.sigma0 };
    int       s    = 0;
    int       t    = 1; // init success and iteration counter

    pstar.g = this.nullary.create(this.random); // create first genotype
    pstar.x = this.gpm.gpm(pstar.g);            // get phenotype for that genotype
    pstar.v = f.compute(pstar.x);               // evaluate: how good is the phenotype?

    while (!(this.termination.shouldTerminate())) { // until we should finish...
      pnew.g = esUnary.mutate(pstar.g, sigma, this.random); // mutate using sigma
      pnew.x = this.gpm.gpm(pnew.g);                        // get phenotype for that genotype
      pnew.v = f.compute(pnew.x);                           // evaluate the new phenotype

      if (pnew.v <= pstar.v) { // if the new individual is better...
        pstar.assign(pnew);    // it becomes the new best individual
        s++;                   // count the success
      }

      if ((t % this.L) == 0) { // is it time to update sigma?
        double Ps = (((double) s) / ((double) (this.L))); // need floating point div! 
        if (Ps < 0.2d) {
          sigma[0] *= this.a;   // not enough success: decrease sigma
        } else {
          if (Ps > 0.2d) {
            sigma[0] /= this.a; // too successful: increase sigma
          } 
        }
        s = 0; // reset success counter
      }

      t++; // count iteration
    }
    return pstar; // return the best individual that we have discovered
  }
}
