package metaheuristicOptimization.examples.symbolicRegression;// start

import metaheuristicOptimization.Individual;
import metaheuristicOptimization.algorithms.ea.EA;
import metaheuristicOptimization.algorithms.ea.selection.TruncationSelection;
import metaheuristicOptimization.searchSpaces.trees.data.NodeTypeSet;
import metaheuristicOptimization.searchSpaces.trees.data.ReflectionNodeType;
import metaheuristicOptimization.searchSpaces.trees.data.math.RealFunction;
import metaheuristicOptimization.searchSpaces.trees.data.math.arith.Abs;
import metaheuristicOptimization.searchSpaces.trees.data.math.arith.Add;
import metaheuristicOptimization.searchSpaces.trees.data.math.arith.Mul;
import metaheuristicOptimization.searchSpaces.trees.data.math.arith.Sub;
import metaheuristicOptimization.searchSpaces.trees.data.math.basic.VariableType;
import metaheuristicOptimization.searchSpaces.trees.data.math.trig.Cos;
import metaheuristicOptimization.searchSpaces.trees.ops.TreeMutator;
import metaheuristicOptimization.searchSpaces.trees.ops.TreeRampedHalfAndHalf;
import metaheuristicOptimization.searchSpaces.trees.ops.TreeRecombination;
import metaheuristicOptimization.termination.MaxSteps;

// end
/** a simple test class applying GP to find a function */
public class SymbolicRegressionGP {// start
  /** Run the test class
   * @param args
   *          the command line arguments */
  @SuppressWarnings("unchecked")
  // end
  public static void main(final String[] args) {
    final EA<RealFunction, RealFunction>      ea      = new EA<>();         // create EA
    final SymbolicRegressionObjective         f;
    final NodeTypeSet<RealFunction>           nts;
    final NodeTypeSet<RealFunction>[]         binary, unary;

    nts    = new NodeTypeSet<>();
    binary = new NodeTypeSet[] { nts, nts }; //a binary function has two child elements
    unary  = new NodeTypeSet[]  { nts };     //a unary function has only one

    nts.add(new ReflectionNodeType<>(Cos.class, unary));  // consine
    nts.add(new ReflectionNodeType<>(Add.class, binary)); // addition
    nts.add(new ReflectionNodeType<>(Sub.class, binary)); // subtraction
    nts.add(new ReflectionNodeType<>(Mul.class, binary)); // multiplication
    nts.add(new ReflectionNodeType<>(Abs.class, unary));  // absolute value |x|
    nts.add(new VariableType(3));                         // 3 variables

    f = new SymbolicRegressionObjective(// data from: cos(x0+x1-x2)
        new DataPoint(new double[] {4.39, 2.55, 3.03}, -0.719018461), //
        new DataPoint(new double[] {3.9, 3.56, 1.72}, 0.856066652), //
        new DataPoint(new double[] {3.16, 0.43, 2.31}, 0.28671521), //
        new DataPoint(new double[] {2.46, 0.94, 0.27}, -0.999932806), //
        new DataPoint(new double[] {2.55, 2.22, 0.69}, -0.591073437), //
        new DataPoint(new double[] {1.2, 2.19, 4.58}, 0.371659872), //
        new DataPoint(new double[] {1.87, 2.36, 4.32}, 0.995952733), //
        new DataPoint(new double[] {3.8, 2.24, 0.29}, 0.861192417), //
        new DataPoint(new double[] {3.2, 0.78, 1.99}, -0.407033207), //
        new DataPoint(new double[] {1.34, 4.43, 1.5}, -0.428099682), //
        new DataPoint(new double[] {1.63, 0.11, 0.8}, 0.589788025), //
        new DataPoint(new double[] {2.94, 2.84, 4.05}, -0.158532001), //
        new DataPoint(new double[] {3.84, 4.72, 2.66}, 0.927478431), //
        new DataPoint(new double[] {3.32, 4.54, 1.36}, 0.976587626), //
        new DataPoint(new double[] {2.89, 3.32, 4.91}, 0.267498829), //
        new DataPoint(new double[] {1.51, 4.58, 3.99}, -0.504846105), //
        new DataPoint(new double[] {2.28, 0.72, 4.17}, 0.390151684), //
        new DataPoint(new double[] {0.08, 1.15, 0.19}, 0.506220257), //
        new DataPoint(new double[] {2.64, 1.08, 1.19}, -0.818734599), //
        new DataPoint(new double[] {1.19, 0.64, 4.59}, -0.92807271), //
        new DataPoint(new double[] {4.69, 2.86, 2.67}, 0.166827326), //
        new DataPoint(new double[] {3.48, 4.8, 2.59}, 0.829164286), //
        new DataPoint(new double[] {2.33, 0.72, 3.13}, 0.996801706), //
        new DataPoint(new double[] {2, 4.51, 3.15}, -0.976243776), //
        new DataPoint(new double[] {4.6, 2.34, 2.4}, -0.171536407), //
        new DataPoint(new double[] {4.53, 1.37, 2.87}, -0.993779999), //
        new DataPoint(new double[] {2.55, 4.99, 3.54}, -0.653643621), //
        new DataPoint(new double[] {4.17, 3.28, 3.59}, -0.752854947), //
        new DataPoint(new double[] {2.59, 0.24, 0.97}, -0.285189059), //
        new DataPoint(new double[] {1.38, 2.5, 4.64}, 0.724836011)//
        );

    ea.nullary     = new TreeRampedHalfAndHalf<>(nts, 6); // Ramped-Half-and-Half: max depth 6
    ea.unary       = new TreeMutator<>(6);                // the tree mutator: max depth 6
    ea.binary      = new TreeRecombination<>(6);          // binary search operation (crossover)
    ea.ps          = 256;                                 // set population size to some good value
    ea.mps         = 128;                                 // set mating pool size to some good value
    ea.selection   = TruncationSelection.INSTANCE;        // use truncation selection
    ea.cr          = 0.3;                                 // set crossover rate to some good value

    for (int i = 1; i <= 100; i++) {           // let's do 100 runs
      ea.termination = new MaxSteps(1000000);  // for each run, allocate 1'000'000 steps
      Individual<RealFunction, RealFunction> res = ea.solve(f); // invoke optimizer

      System.out.println("run " + i + //$NON-NLS-1$
          " has result quality " + res.v +
          " and result " + res.x.toString()); //$NON-NLS-1$
    }
  }
}
