package metaheuristicOptimization.examples.symbolicRegression;

import metaheuristicOptimization.IObjectiveFunction;
import metaheuristicOptimization.searchSpaces.trees.data.math.RealFunction;

/** an objective function for symbolic regression which returns the root-mean-square-error */
public class SymbolicRegressionObjective implements IObjectiveFunction<RealFunction> {
  /** the data */
  private final DataPoint[] data;

  /**
   * create
   * 
   * @param d
   *          the data
   */
  public SymbolicRegressionObjective(final DataPoint... d) {
    super();
    this.data = d;
  }

  /** {@inheritDoc} */
  @Override
  public double compute(final RealFunction x) {
    double error, diff;
    error = 0d;
    for (DataPoint d : this.data) {
      diff = x.compute(d.input) - d.output;
      error += (diff * diff);
    }
    return Math.sqrt(error / this.data.length);
  }
}
