// start
package metaheuristicOptimization.searchSpaces.Rn;

import java.util.Random;

import metaheuristicOptimization.IUnarySearchOperation;

/** a special unary search operator designed for evolutionary strategies */
// end
public class RnESUnaryNormal extends Rn implements IUnarySearchOperation<double[]> {// start

  /** a supplement standard dev */
  private static final double[] DEV_STDDEV = new double[] {
    0.001d };

  /**
   * instantiate this operation for a given subspace of the Rn
   * 
   * @param def
   *          the subspace
   */
  public RnESUnaryNormal(final Rn def) {
    super(def);
  }

  /**
   * Perform mutation with a genotype and a parameter holding standard deviations
   * 
   * @param genotype
   *          the parent
   * @param sigma
   *          the std dev vector
   * @param r
   *          the randomizer
   * @return the new vector
   */
  // end
  public double[] mutate(final double[] genotype, final double[] sigma, final Random r) {
    double d;

    double[] g = genotype.clone(); // copy the original vector

    for (int i = g.length; (--i) >= 0;) {
      do { // create a value close to that gene by using the step length parameter
        d = (g[i] + (r.nextGaussian() * sigma[i % sigma.length]));
      } while ((d < this.min) || (d > this.max)); // make sure that value is OK

      g[i] = d; // store value into copied genotype
    }

    return g; // return the modified copy of the original genotype
  }// start

  /** {@inheritDoc} */
  @Override
  public double[] mutate(final double[] genotype, final Random r) {
    return this.mutate(genotype, DEV_STDDEV, r);
  } // end
}
