package metaheuristicOptimization.searchSpaces.trees.data.math;

import metaheuristicOptimization.searchSpaces.trees.data.Node;
import metaheuristicOptimization.searchSpaces.trees.data.NodeType;

/** A mathematics function which works one a vector of parameters and returns a single
 * value, as used, for instance, in Symbolic Regression Section 49.1.
 * @author Thomas Weise */
public class RealFunction extends Node<RealFunction> {
  /** a constant required by Java serialization */
  private static final long serialVersionUID = 1;

  /** Create a node with the given children
   * @param pchildren
   *          the child nodes
   * @param in
   *          the node information record */
  public RealFunction(final Node<?>[] pchildren, final NodeType<? extends RealFunction> in) {
    super(pchildren, in);
  }

  /** This is the core method of mathematics functions: it computes the result of this
   * function and can be invoked recursively downwards for the other nodes.
   * @param data
   *          the context containing the data vector
   * @return the result of the computation */
  public double compute(final double[] data) {
    return 0d;
  }

  /** Print a sub-expression to the given string builder
   * @param idx
   *          the sub-expression index
   * @param sb
   *          the string builder */
  protected final void printSubExpression(final int idx, final StringBuilder sb) {
    RealFunction a;
    a = this.get(idx);
    sb.append('(');
    a.fillInText(sb);
    sb.append(')');
  }
}