package metaheuristicOptimization.searchSpaces.trees.data.math.arith;

import metaheuristicOptimization.searchSpaces.trees.data.Node;
import metaheuristicOptimization.searchSpaces.trees.data.NodeType;
import metaheuristicOptimization.searchSpaces.trees.data.math.RealFunction;

/** Divide two numbers
 * @author Thomas Weise */
public class Div extends RealFunction {
  /** a constant required by Java serialization */
  private static final long serialVersionUID = 1;

  /** Create a node with the given children
   * @param pchildren
   *          the child nodes
   * @param in
   *          the node information record */
  public Div(final Node<?>[] pchildren, final NodeType<Div> in) {
    super(pchildren, in);
  }

  /** Compute the value
   * @param a
   *          the first number
   * @param b
   *          the second number
   * @return the result of the division */
  private static final double compute(final double a, final double b) {
    double d;
    d = (a / b);
    if (Double.isNaN(d)) { return 1d; }
    return d;
  }

  /** Divide two values
   * @param data
   *          the context containing the data vector
   * @return the result of the computation */
  @Override public double compute(final double[] data) {
    return Div.compute(this.get(0).compute(data), this.get(1).compute(data));
  }

  /** Fill in the text associated with this node
   * @param sb
   *          the string builder */
  @Override public void fillInText(final StringBuilder sb) {
    this.printSubExpression(0, sb);
    sb.append('/');
    this.printSubExpression(1, sb);
  }
}