package metaheuristicOptimization.searchSpaces.trees.data.math.arith;

import metaheuristicOptimization.searchSpaces.trees.data.Node;
import metaheuristicOptimization.searchSpaces.trees.data.NodeType;
import metaheuristicOptimization.searchSpaces.trees.data.math.RealFunction;

/** Compute the exponent of a given number
 * @author Thomas Weise */
public class Exp extends RealFunction {
  /** a constant required by Java serialization */
  private static final long serialVersionUID = 1;

  /** Create a node with the given children
   * @param pchildren
   *          the child nodes
   * @param in
   *          the node information record */
  public Exp(final Node<?>[] pchildren, final NodeType<Exp> in) {
    super(pchildren, in);
  }

  /** Compute the exponent of a given number
   * @param data
   *          the context containing the data vector
   * @return the result of the computation */
  @Override public double compute(final double[] data) {
    return Math.exp(this.get(0).compute(data));
  }

  /** Fill in the text associated with this node
   * @param sb
   *          the string builder */
  @Override public void fillInText(final StringBuilder sb) {
    sb.append("exp("); //$NON-NLS-1$
    this.get(0).fillInText(sb);
    sb.append(')');
  }
}