package metaheuristicOptimization.searchSpaces.trees.data.math.arith;

import metaheuristicOptimization.searchSpaces.trees.data.Node;
import metaheuristicOptimization.searchSpaces.trees.data.NodeType;
import metaheuristicOptimization.searchSpaces.trees.data.math.RealFunction;

/** Frac a given number
 * @author Thomas Weise */
public class Frac extends RealFunction {
  /** a constant required by Java serialization */
  private static final long serialVersionUID = 1;

  /** Create a node with the given children
   * @param pchildren
   *          the child nodes
   * @param in
   *          the node information record */
  public Frac(final Node<?>[] pchildren, final NodeType<Frac> in) {
    super(pchildren, in);
  }

  /** Floor a number
   * @param d
   *          the number
   * @return the result */
  private static final double compute(final double d) {
    double a;
    a = (d - Math.floor(d));
    if (Double.isNaN(a)) { return 0d; }
    return a;
  }

  /** Compute the floor value of a given number
   * @param data
   *          the context containing the data vector
   * @return the result of the computation */
  @Override public double compute(final double[] data) {
    return Frac.compute(this.get(0).compute(data));
  }

  /** Fill in the text associated with this node
   * @param sb
   *          the string builder */
  @Override public void fillInText(final StringBuilder sb) {
    sb.append("frac"); //$NON-NLS-1$
    this.printSubExpression(0, sb);
  }
}