package metaheuristicOptimization.searchSpaces.trees.data.math.arith;

import metaheuristicOptimization.searchSpaces.trees.data.Node;
import metaheuristicOptimization.searchSpaces.trees.data.NodeType;
import metaheuristicOptimization.searchSpaces.trees.data.math.RealFunction;

/** Compute the logarithm of a given number
 * @author Thomas Weise */
public class Log extends RealFunction {
  /** a constant required by Java serialization */
  private static final long serialVersionUID = 1;

  /** Create a node with the given children
   * @param pchildren
   *          the child nodes
   * @param in
   *          the node information record */
  public Log(final Node<?>[] pchildren, final NodeType<Log> in) {
    super(pchildren, in);
  }

  /** Compute
   * @param a
   *          the constant
   * @return the value */
  private static final double compute(final double a) {
    double d;
    d = a;
    if (d < 0d) {
      d = (-d);
    }
    d = Math.log(d);
    if (Double.isNaN(d)) { return 1d; }
    return d;
  }

  /** Compute the square root of a given number
   * @param data
   *          the context containing the data vector
   * @return the result of the computation */
  @Override public double compute(final double[] data) {
    return Log.compute(this.get(0).compute(data));
  }

  /** Fill in the text associated with this node
   * @param sb
   *          the string builder */
  @Override public void fillInText(final StringBuilder sb) {
    sb.append("log"); //$NON-NLS-1$
    this.printSubExpression(0, sb);
  }
}