package metaheuristicOptimization.searchSpaces.trees.data.math.ctrl;

import metaheuristicOptimization.searchSpaces.trees.data.Node;
import metaheuristicOptimization.searchSpaces.trees.data.NodeType;
import metaheuristicOptimization.searchSpaces.trees.data.math.RealFunction;

/** If-Then-Else
 * @author Thomas Weise */
public class IfThenElse extends RealFunction {
  /** a constant required by Java serialization */
  private static final long serialVersionUID = 1;

  /** Create a node with the given children
   * @param pchildren
   *          the child nodes
   * @param in
   *          the node information record */
  public IfThenElse(final Node<?>[] pchildren, final NodeType<IfThenElse> in) {
    super(pchildren, in);
  }

  /** IfThenElse two values
   * @param data
   *          the context containing the data vector
   * @return the result of the computation */
  @Override public double compute(final double[] data) {
    if (this.get(0).compute(data) > 0d) { return this.get(1).compute(data); }
    return this.get(2).compute(data);
  }

  /** Fill in the text associated with this node
   * @param sb
   *          the string builder */
  @Override public void fillInText(final StringBuilder sb) {
    sb.append("if "); //$NON-NLS-1$
    this.printSubExpression(0, sb);
    sb.append(">0 then ");//$NON-NLS-1$
    this.printSubExpression(1, sb);
    sb.append(" else ");//$NON-NLS-1$
    this.printSubExpression(2, sb);
  }
}