package metaheuristicOptimization.searchSpaces.trees.data.math.trig;

import metaheuristicOptimization.searchSpaces.trees.data.Node;
import metaheuristicOptimization.searchSpaces.trees.data.NodeType;
import metaheuristicOptimization.searchSpaces.trees.data.math.RealFunction;

/** Compute the arcus sinus of a number
 * @author Thomas Weise */
public class ASin extends RealFunction {
  /** a constant required by Java serialization */
  private static final long serialVersionUID = 1;

  /** Create a node with the given children
   * @param pchildren
   *          the child nodes
   * @param in
   *          the node information record */
  public ASin(final Node<?>[] pchildren, final NodeType<ASin> in) {
    super(pchildren, in);
  }

  /** Compute
   * @param a
   *          the value
   * @return the result */
  private static final double compute(final double a) {
    double d;
    d = a;
    if (Math.abs(a) > 1d) {
      d = (1d / d);
    }
    d = Math.asin(d);
    if (Double.isNaN(d)) { return 1d; }
    return d;
  }

  /** Compute the tangent
   * @param data
   *          the context containing the data vector
   * @return the result of the computation */
  @Override public double compute(final double[] data) {
    return ASin.compute(this.get(0).compute(data));
  }

  /** Fill in the text associated with this node
   * @param sb
   *          the string builder */
  @Override public void fillInText(final StringBuilder sb) {
    sb.append("asin"); //$NON-NLS-1$
    this.printSubExpression(0, sb);
  }
}