package metaheuristicOptimization.searchSpaces.trees.ops;

import java.util.Random;

import metaheuristicOptimization.INullarySearchOperation;
import metaheuristicOptimization.searchSpaces.trees.data.Node;
import metaheuristicOptimization.searchSpaces.trees.data.NodeTypeSet;

/**
 * A tree creator using the ramped-half-and-half method as described in Section 31.3.2.3.
 *
 * @param <NT>
 *          the node type
 * @author Thomas Weise
 */
public class TreeRampedHalfAndHalf<NT extends Node<NT>> extends TreeOperation implements
    INullarySearchOperation<NT> {
  /** a constant required by Java serialization */
  private static final long serialVersionUID = 1;

  /** the types to choose from */
  public final NodeTypeSet<NT> types;

  /**
   * Create a new ramped-half-and-half
   *
   * @param md
   *          the maximum tree depth
   * @param ptypes
   *          the types
   */
  public TreeRampedHalfAndHalf(final NodeTypeSet<NT> ptypes, final int md) {
    super(md);
    this.types = ptypes;
  }

  /**
   * Create a new tree according to the ramped-half-and-half method given in Algorithm 31.3.
   *
   * @param r
   *          the random number generator
   * @return a new genotype (see Definition D4.2)
   */
  @Override
  public NT create(final Random r) {
    return TreeOperation.createTree(this.types,//
        (2 + r.nextInt(this.maxDepth)), r.nextBoolean(), r);
  }
}